function result = iPatch(pdbfile1, pdbfile2, MSAfile, location, segments, jobname, parameters)

%% INTER-PROTEIN CONTACT PREDICTION BASED ON LOCAL PATCH INFORMATION -- iPatch
% This program is used to predict the inter-protein (domain)
% contact sites by using the amino acid propensity, pair
% propensity, triangle propensity of the interface, as well as the
% structure information for each contacting partner.

%% REQUIRMENT
% Unix with PSA: if you want to run it on windows, you need to change the
% code that calling PSA from matlab in JoyInformation.m. Currently, it is
% runUnix. It can be replaced with runDos when your PSA is installed in windows.
% this programe requires PSA installed on Unix to calculate the solvent accessable sureface aera for
% each protein. This programe can be found in the following link:
% the psa excutable file has already been included in the package ./data/
% BUG: some times the calling of PSA from matlab fails with some unknown reason
% and the whole matlab progrme will be terminated, please simply restart
% it and run it again.

%% INPUT
% pdbfile ---- two pdbfiles for two proteins. e.g., {'yourprotein1.pdb',
%              'yourprotein2.pdb'}
% MSAfile ---- this is the multiply sequence alignment for the paired 
%              protein sequences in FASTA formart. In the MSA, the
%              reference sequence must be specified by the header
%              '>reference'.
% segments.number  ---- the residue sequence numbers 
%               .chain   ----   the chain ID 
%                                 of the boundaries for the interested segments on the
%                                  reference sequence.Currently, only two segments are
%                                 allowed, and noncontinuous segments are also allowed.
% location ----  the start sites and end sites of the domains of interest
%                   on the space free reference sequence. It has two cell
%                   data with the location vectors for two domains. E.g.,
%                   location{1} = [1,10; 50, 100];    
%                   location{2} = [20, 49];
%                   the number 1 is the location of the starting residue of the first domain,
%                   the number 10 is the location of the end of the first
%                   segment of the first domain. and these numbers do not
%                   take any gap into account.          
% jobname ---- the name of your complex, it will be in the names of many cache
%             files
% parameters ----  (the default settings are showing)
%            parameters.surfaceoption = 1; % the surface information for the domain interested is
%                               % given by running the psa on the domain; if set it to
%                               % 1, the surface information for the domain interested is
%                               % given by running the psa on the whole protein on
%                               % which this domain is located;
%            parameters.gapcutoff = 0.5;   % columns with gap percentage greater than this cutoff wont be used in the score calculation
%            parameters.discutoff = 4.5;   % the intra domain space neighbors are defined by this cutoff, i.e., if the distances 
%                               % between residues that are measured as the
%                               % shortest distance between any nonhydrogen atoms
%                               % are less than this cutoff, these two
%                               % residues are intra domain neighbors
%            parameters.parallelized  % 1, if you can run the matlabpool, most of the algorithms can be
%                               % parallelized; 0, otherwise.
%            parameters.randomized = 1;    % 1, if you want to get the scores for the random matching pairs as backgroud score; 0, otherwise.
%            parameters.reference = 0;     % 1, if you want to include your reference sequence in the score calculation; 0, otherwise.  
%            parameters.cleancache = 1;    % clean all the cache files; if set to be 0, the cache files will be used, if any

%% IMPORTNAT NOTES:
% 1) MSA
% recommend to do the multiple alignment for each protein family
% separately by muscle-maxalign-muscle concatenat the sequences 
% of the contact pairs by using any pairing information you have
% and make sure only one-to-one match, and then delete the redundancy.
% 2) pdbfile
% the pdb file will be read into a structure by using pdbread (matlab
% function), and only the coordinates in the ATOM section will be used
% which means those modified residues in the pdb file will be ignored
% 3) Distance
% the distance between residues are calculated as the smallest all atom
% distance, which is implemented in the function: compute_atom_distances.m
% if you need to calculate the distance only between side chain atom,
% please commend the lines 26 ~ 28, and uncommend the lines 14 ~ 25.
% 4) parameter settings
% some parameters about the GAP cutoff, distance cutoff, parallelized or
% not, unix or windows can be modified, calclulate the score for randomized
% pairings or not
% 5) CACHE
% there are many CACHE codes you could uncommand them, if you are running 
% this for the same proteins
% 6) parallel
% if matlabpool is available, parameters.parallelized = 1; (default)


%% Output
% result.pairwisescores{1} pairPPro  
% result.pairwisescores{2} pairTPro  
% result.pairwisescores{3} pairMcBASC  
% result.sitescores{1} APro 
% result.sitescores{2} PPro
% result.sitescores{3} TPro
% result.sitescores{4} EBMcBASC
% result.sitescores{5} Comb 
% result.protein  a structure data format with many fields


%% %%%%%%%%%%%%%%%%%%%%parmeter settings%%%%%%%%%%%%%%%%%%%%%%%%%%
% Unix
resultpath = [pwd,'/result/'];
datapath = [pwd, '/data/'];
temppath = [pwd, '/processing/'];
temppathrandom = [temppath, '/random_processing/'];

% % Windows
% resultpath = [pwd, '\result\'];
% datapath = [pwd, '\data\'];
% temppath = [pwd, '\processing\'];
% temppathrandom = [temppath, '/random_processing/'];

%% clean the cache
if parameters.cleancache
    runUnix(resultpath, ['rm -f *',jobname,'*']);
    runUnix(temppath, ['rm -f *',jobname,'*']);
    runUnix(temppathrandom, ['rm -f *',jobname,'*']);
end

%% deal with the modified residues in pdb file 
if parameters.modifiedres
   bool = changeHETATM2ATOMinpdbForMSE([datapath, pdbfile1], [datapath, 'temp', pdbfile1]); 
   if bool == 1
       disp(['Modified residues involved in', pdbfile1]);
       disp(['This pdb file has been edited, and the old one is', pdbfile1, '.backup']);
       runUnix(datapath, ['mv ', pdbfile1, ' ', pdbfile1, '.backup']);
       runUnix(datapath, ['mv temp', pdbfile1, ' ', pdbfile1]);
   end
   bool = changeHETATM2ATOMinpdbForMSE([datapath, pdbfile2], [datapath, 'temp', pdbfile2]);
   if bool == 1
       disp(['Modified residues involved in', pdbfile2]);
       disp(['This pdb file has been edited, and the old one is', pdbfile2, '.backup']);
       runUnix(datapath, ['mv ', pdbfile2, ' ', pdbfile2, '.backup']);
       runUnix(datapath, ['mv temp', pdbfile2, ' ', pdbfile2]);
   end 
end

%% %%%%%%%%%%%%%%%%%%%%%%%%%%protein preparing%%%%%%%%%%%%%%%%%%%%
'protein structure preparing ...'
fidp = fopen([resultpath, jobname,'.mat']);
if fidp ~= -1
    load([resultpath, jobname,'.mat']);
else
    % input read
    clear protein;
    protein.names = jobname;
    protein.gi_number = 'reference';
    protein.alnfile_name = [datapath, MSAfile];
    protein.infile = [temppath, jobname,'_nogap.fasta'];
    protein.infile_randomized = [temppath, jobname, '_randomized.fasta'];
    protein.pdbfiles = {pdbfile1, pdbfile2};
    protein.domain = segments;

    % read sequence from MSA file
    protein.aln.seq = fastaread(protein.alnfile_name);
    protein.aln.length = size(protein.aln.seq(1).Sequence, 2);

    % find the location of each residue in the MSA ignoring the gaps
    protein.aln.locationMSA = find_location(protein.aln.seq, protein.gi_number);
    clear temp;
    temp = find_sequence(protein.aln.seq, protein.gi_number);
    referenceseq = temp(protein.aln.locationMSA); % without spaces
    clear temp;

    % assign residue sequence numbers for the residues on the reference sequence
    % by referring to corresponding pdb files
    clear pdbstructure;
    for i = 1 : 2
        pdbstructure{i} = pdbread([datapath, protein.pdbfiles{i}]);
    end
    % the residue sequence numbers for every residue on the reference
    % sequence
%     save('temp.mat', 'pdbstructure', 'referenceseq', 'protein',...
%         'location', 'parameters', 'jobname')
    protein.residue = assign_resSeq2refseq(pdbstructure, referenceseq, protein.domain,...
        location, parameters.manualalignment, jobname);
    
    %% change the chain ID to A and B for two proteins
    %% and put the residue numbers on to the sequence according to the
    %% domain locations
    protein.aln.domain = protein.domain;
    protein.aln.domain.chain = {'A','B'};
    res_location = [];
    for i  = 1 : 2
        for j = 1 : size(location{i},1)
            res_location = [res_location, location{i}(j,1) : location{i}(j,2)];
        end
    end
    t = 1;
    for i = 1 : 2
        for j = 1 : size(protein.residue{i}.number,2)            
            protein.aln.residue.number(res_location(t)) = protein.residue{i}.number(j);
            if i == 1
                protein.aln.residue.chain(res_location(t)) = 'A';
            else
                protein.aln.residue.chain(res_location(t)) = 'B';
            end
            t = t + 1;
        end
    end

    % make a new pdb file for all the coordinates
    'cleaning and making a newpdb file...'
    newpdbfilename = ['newpdb',protein.names,'.pdb'];
    fidd = fopen([temppath, newpdbfilename],'r');
    if fidd == -1
        makemypdbfile(protein, referenceseq, newpdbfilename, datapath, temppath);
    else
        fclose(fidd);
        'already in the cache'
    end
    'Done!'
    protein.structure = pdbread([temppath, newpdbfilename]);

    % make a new aln file for the method
    % 1. put the columns for one domain together
    % 2. delete the columns with too many gaps
    % 3. calculate the intradomain distances and generate the neighbor
    % information
    %save('res.mat', 'protein','parameters','temppath');
    protein = preprocess_aln_file(protein, parameters, temppath);
    %% randomize the aln pairs except for the reference sequence
    if parameters.randomized
        protein = randomizePairing(protein);
    end
    save([resultpath, protein.names,'.mat'], 'protein');
    'protein data saved.'
end
%% %%%%%%%%%%%%%%%%%%%%%%%%%joy information%%%%%%%%%%%%%%%%%%%%%%%
'run joy, get surface information...'
fiddata = fopen([resultpath, protein.names, 'joyfeatures.mat']);
if fiddata == -1    
    clear joyinformation;
    switch parameters.surfaceoption
        case 0
            % run psa on each domain
            joyinformation = JoyInformation(protein, temppath);
        case 1
            % run psa on each protein
            joyinformation = JoyInformationProtein(protein, datapath);
    end
    EBindex = joyinformation.EBindex;
    joypsa = joyinformation.joypsa;
    save([resultpath, protein.names, 'joyfeatures.mat'], 'EBindex','joypsa');
    clear joyinformation;    
else
    fclose(fiddata);
    load([resultpath, protein.names, 'joyfeatures.mat']);
end
%% if you do not want to include the reference sequence into score
%% calculation
if ~parameters.reference
    fullprotein = protein;
    protein = notincludereference(fullprotein, parameters.randomized);   
end   

'Done!'
%% %%%%%%%%%%%%%%%%%%%%%%%%%calculate scores%%%%%%%%%%%%%%%%%%%%%%
clear temp;
temp = load('statsonalldata.mat');
stats = temp.statsonbigdata;
clear temp;
clear tallscores;
if parameters.parallelized
    if matlabpool('size') > 0
    else
        matlabpool 
    end
end
tallscores = claclulaallscores_final(protein, EBindex, stats, resultpath, temppath, parameters);
%% if you want to calculate the scores for the randomized pairing as the
%% background score
if parameters.randomized
    tallscores_random = claclulaallscores_randomized(protein, EBindex, stats, resultpath, temppathrandom, parameters);
end

if parameters.parallelized
    if matlabpool('size') > 0
        matlabpool close
    end
end
allmodels = [1 1 1 1];
fits.coeff{1} = [-5.5267 3.2675 -3.118 3.0213 2.1648]';
fitscores = CalculateFitmodels_final (protein, EBindex, tallscores.scores, fits, allmodels);
%% if you want to calclulate the scores for the randomized pairing as the
%% background score
if parameters.randomized
    fitscores_random = CalculateFitmodels_final (protein, EBindex, tallscores_random.scores, fits, allmodels);
end

% output the result
clear result;
result.pairwisescores{1} = tallscores.pairscores{1};  
result.pairwisescores{2} = tallscores.pairscores{2};  
result.pairwisescores{3} = tallscores.pairscores{3};   
result.sitescores{1} = tallscores.scores{1};
result.sitescores{2} = tallscores.scores{2};
result.sitescores{3} = tallscores.scores{3};
result.sitescores{4} = tallscores.scores{4};
result.sitescores{5} = fitscores{1};
result.scorenames = tallscores.scorenames;
result.scorenames{5} = 'Comb'; 
result.pairscorenames = tallscores.pairscorenames;
if ~parameters.reference
    result.protein = fullprotein;
else   
    result.protein = protein;
end   
if parameters.randomized
    result.random.pairwisescores{1} = tallscores_random.pairscores{1};
    result.random.pairwisescores{2} = tallscores_random.pairscores{2};
    result.random.pairwisescores{3} = tallscores_random.pairscores{3};
    result.random.sitescores{1} = tallscores_random.scores{1};
    result.random.sitescores{2} = tallscores_random.scores{2};
    result.random.sitescores{3} = tallscores_random.scores{3};
    result.random.sitescores{4} = tallscores_random.scores{4};
    result.random.sitescores{5} = fitscores_random{1};
    result.random.scorenames = tallscores_random.scorenames;
    result.random.scorenames{5} = 'Comb';
    result.random.pairscorenames = tallscores_random.pairscorenames;
end

%% output scores to file
outputscores(result,resultpath);
%% save the result
save([resultpath, protein.names, 'result.mat'], 'result');

